palette = [
    ('body', 'white', 'black'),
    ('session', 'light gray', 'black'),
    ('session_focus', 'black', 'light green', 'standout'),
    ('folder', 'light blue', 'black', 'bold'),
    ('folder_focus', 'black', 'light green', 'bold'),
    ('head', 'yellow', 'black', 'standout'),
    ('foot', 'light gray', 'black'),
    ('key', 'light cyan', 'black', 'underline'),
    ('title', 'white', 'black', 'bold'),
    ('dirmark', 'black', 'dark cyan', 'bold'),
    ('flag', 'dark gray', 'light gray'),
    ('error', 'light red', 'black'),
    ('fbutton', 'white', 'dark gray'),
    ('fbutton_sc', 'light green', 'black', 'bold'),
    ('dialog', 'white', 'black'),
    ('dialog_button', 'black', 'light green'),
    ('dialog_button_focus', 'white', 'dark green', 'bold'),
    ('dialog_edit', 'black', 'light gray'),
    ('dialog_edit_focus', 'black', 'yellow'),
    ('ssh_copy', 'black', 'dark green'),
    ('ssh_copy_focus', 'white', 'dark green', 'bold'),
    #(name, foreground, background, mono, foreground_high, background_high)
]
